{% extends "base.html" %}

{% block main %}
<a href="/">&laquo; home</a>
    <h2>Archives for  {{ year }}</h2>
{% for post in posts %}
    <div class="post">
        <h3><a href="/{{ post.url }}">{{ post.config.title }}</a></h3>
    </div>
{% endfor %}
{% endblock %}
