<!DOCTYPE html>
<html>
<head>
    <title>{% block title %}{% endblock %} - {{ site_name }}</title>
    <meta charset="utf-8" />
    <link rel="stylesheet" href="/media/style.css" type="text/css" media="screen" />
</head>
<body>
<div id="wrap">
    <div id="header">
        <h1>{{ site_name }}</h1>
    </div>

    {% block main %}
        
    {% endblock %}

    <div id="footer">
        <p>
            <strong>Categories:</strong>
            {% for cat in categories %}
            <a href="/category/{{ cat.slug }}/">{{ cat.name }}</a>{% if not forloop.last %}, {% endif %}
            {% endfor %}
        </p>
        <p>
            <strong>Archives:</strong>
            {% for y in years %}
            <a href="/archive/{{ y }}/">{{ y }}</a>{% if not forloop.last %}, {% endif %}
            {% endfor %}
        </p>
        <p>&copy; 2011-2012 - All rights reserved.</p> 
    </div>
</div>
</body>
</html>
